'use strict';

var through = require('through2');
var path = require('path');
var fs = require('fs');
var uglifyjs = require('uglify-js');
var replacepath = require('replacepath');
var rev = require('./rev');
var mod = require('./module');
var crypto = require('crypto');

var createHash = function(file, len){
    return crypto.createHash('md5').update(file).digest('hex').substr(0, len);
}

var unique = function(arr){
    var newarr = [];
    var temp = {};
    arr.forEach(function(val){
        if(!temp[val]){
            temp[val] = true
            newarr.push(val)
        }
    })
    return newarr
}

var basePath = path.join(__dirname, '../../');

module.exports = function(opts){
    var options = {
        //项目根目录，修改paths中base的值
        base:'',
        //配置文件路径
        url:'',
        //路径别名
        paths:{},
        //模块别名
        alias:{},
        //忽略的模块名，模块不会追加到合并的文件中
        ignore:['util', 'template', 'component', 'events'],
        //版本号长度
        verlen:7,
        //是否在配置文件中生成版本映射
        vermap:true,
        //生成合并css压缩文件
        cssmin:true,
        //生成合并js文件
        jsdebug:true,
        //生成合并js压缩文件
        jsmin:true,
        //过滤资源文件路径，去除或替换一些特殊符号，返回绝对路径
        filterPath:null,
        //gulp-watch返回的对象
        watcher:null,
        //只输出改变过的文件
        changed:true
    }

    var configContent, configObject;

    if(opts){
        if(opts.url && fs.existsSync(opts.url)){
            var buffer = fs.readFileSync(opts.url);
            var content = buffer.toString();
            var match = content.match(/Nui.config(\(\{((.|\s)*?)\}\))/);
            if(match){
                if(typeof opts.vermap === 'undefined' || opts.vermap === true){
                    configObject = eval(match[1]);
                    configObject.maps = {};
                    configContent = content;
                }
                match = eval(match[1]);
                opts.paths = match.paths || {};
                opts.alias = match.alias || {};
            }
        }

        for(var i in opts){
            options[i] = opts[i]
        }
    }

    var paths = options.paths;
    paths.base = replacepath(options.base || paths.base || basePath).replace(/\\/g, '/')
    for(var i in options.paths){
        var _path = options.paths[i];
        if(i !== 'base' && !/^[A-Z]:/.test(_path)){
            options.paths[i] = paths.base + '/' + _path;
        }
    }

    if(options.jsmin === true){
        options.jsmin = {}
    }

    if(typeof options.jsmin === 'object'){
        if(!options.jsmin.mangle){
            options.jsmin.mangle = {}
        }
        if(!options.jsmin.output){
            options.jsmin.output = {}
        }
        var except = options.jsmin.mangle.except;
        if(!Array.isArray(except)){
            except = []
        }
        options.jsmin.mangle.except = except.concat(['require', 'imports', 'renders', 'extend']);
        options.jsmin.output.keep_quoted_props = true;
        options.jsmin.output.quote_style = 1;
    }

    return through.obj(function(file, enc, cb){

        if(file.isNull()){
	        return cb();
        }

        var dirname = path.dirname(file.path)+'/';
        var content = file.contents.toString().replace(/^\s+|\s+$/g, '');

        if(!content){
            return cb();
        }

        var loads = content.match(/Nui.load\(['"][^'"]+['"]/g);
        var matchs = content.match(/Nui.(load|use)\(['"][^'"]+['"]/g);
        var Module = mod(dirname, options);
        if(loads){
            loads.forEach(function(val, i){
                var id = loads[i].replace(/Nui.load\(/, '').replace(/['"]/g, '').replace(/-min/g, '');
                Module.load(id, function(mod){
                    var contents = unique(mod.getContents()).join('\n');
                    var url;
                    var filename;
                    for(var i in mod.depmodules){
                        url = mod.depmodules[i].url;
                        filename = mod.depmodules[i].filename
                        break;
                    }
                    if(fs.existsSync(url)){
                        var _path = path.join(path.dirname(url), '../style/');
                        var styles = Module.getStyles(unique(mod.loadstyles()), _path).join('\n');
                        if(styles){
                            if(!fs.existsSync(_path)){
                                fs.mkdirSync(_path)
                            }
                            var cssmin = _path + filename + '-min.css';
                            if(options.cssmin === true){
                                styles = styles.replace(/\s*([\}\{\:\;\(\=\~\>\+\[\]\,])\s*/g, '$1')
                                               .replace(/\/\*[^\/]*\*\//g, '')
                                               .replace(/\s+/g, ' ');
                                fs.writeFileSync(cssmin, styles);
                            }
                            else if(fs.existsSync(cssmin)){
                                fs.unlinkSync(cssmin)
                            }
                        }

                        var jsmin = url.replace(/(\.js)$/, '-min$1');
                        var jsdebug = path.basename(url.replace(/(\.js)$/, '-debug$1'));
                        var sourcemap = jsmin+'.map';
                        if(options.jsdebug !== true && fs.existsSync(sourcemap)){
                            fs.unlinkSync(sourcemap);
                        }
                        if(typeof options.jsmin === 'object'){
                            var result;
                            try{
                                if(options.jsdebug === true){
                                    options.jsmin.outSourceMap = path.basename(sourcemap) + '?v='+createHash(new Buffer(contents), options.verlen || 7);
                                    options.jsmin.fromString = contents;
                                    options.jsmin.sourceMapIncludeSources = true;
                                    var files = {};
                                    files[jsdebug] = contents;
                                    result = uglifyjs.minify(files, options.jsmin);
                                    if(result.map){
                                        fs.writeFileSync(sourcemap, result.map);
                                    }
                                }
                                else{
                                    result = uglifyjs.minify(contents, options.jsmin);
                                }
                                if(result){
                                    contents = result.code;
                                    fs.writeFileSync(jsmin, contents);
                                }
                            }
                            catch(e){
                                console.log(e)
                            }
                        }
                        else if(fs.existsSync(jsmin)){
                            fs.unlinkSync(jsmin)
                        }
                    }
                })
            })
        }

        if(options.verlen > 0){
            if(matchs && configObject){
                Module.maps = configObject.maps;
                Module.configContent = configContent;
            }
            var newcon = rev.call(Module, content, dirname, options);
            if(matchs && configContent){
                configObject.maps = Module.maps;
                var _newcontent = configContent.replace(/Nui.config(\(\{((.|\s)*?)\}\))/, function(all, inner, str){
                    return 'Nui.config('+ JSON.stringify(configObject, null, 4) +')'
                })
                fs.writeFileSync(options.url, _newcontent);
            }
            if(options.changed === true && newcon === content){
                return cb()
            }
            file.contents = new Buffer(newcon);
        }

        this.push(file);
        cb();
    })
}